SystemOrganization addCategory: #'FuelFameExtension-Tests'!
SystemOrganization addCategory: #'FuelFameExtension-Core'!

!FLSerializer class methodsFor: '*FuelFameExtension' stamp: 'MartinDias 2/22/2011 23:50'!
on: aStream serialize: entities describedWith: aMetamodel
	"Serializes the Fame entities on aStream"

	| bufferStream |
	bufferStream := ByteArray new writeStream.

	(self on: bufferStream)
		serialize: entities
		describedWith: aMetamodel.

	aStream nextPutAll: bufferStream contents.			! !

!FLSerializer methodsFor: '*FuelFameExtension' stamp: 'MartinDias 2/22/2011 23:50'!
serialize: entities describedWith: aMetamodel
	"Serializes the Fame entities on the stream"

	FLFameObjectClusterSerializer instance repository: 
		((FMRepository with: aMetamodel) 
			addAll: entities;
			yourself).
	self
		serialize: entities
		analyzer: FLFameAnalyzer newDefault.
			! !

FLObjectAnalyzer subclass: #FLFameObjectAnalyzer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Core'!

!FLFameObjectAnalyzer commentStamp: '<historical>' prior: 0!
I am an ObjectAnalyzer for objects whose class is described in Famix.

I work together with FLFameAnalyzer.!

!FLFameObjectAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 11/17/2010 00:03'!
analyzes: anObject

	FLFameObjectClusterSerializer instance repository metamodel 
		descriptionOf: anObject class 
		ifAbsent: [ ^false ].
	^ true! !

!FLFameObjectAnalyzer methodsFor: 'evaluating' stamp: 'MartinDias 11/24/2010 11:30'!
value: anObject

	| serializer |
	self flag: #todo. "Improve this way of configuring the new cluster serializer!!"
	serializer := 
		(FLFameObjectClusterSerializer for: anObject class)
			repository: FLFameObjectClusterSerializer instance repository.	

	analyzer 
		clusters: analyzer clusteredFameObjects 
		add: anObject 
		serializer: serializer! !

FLAnalyzer subclass: #FLFameAnalyzer
	instanceVariableNames: 'clusteredFameObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Core'!

!FLFameAnalyzer commentStamp: '<historical>' prior: 0!
I am an analyzer with a specific graph traversing behavior. 

Given an object described in a Fame repository, I work with its Famix metamodel description, for know which references traverse.!

!FLFameAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 11/23/2010 19:29'!
classDependenciesOf: aClass

	self flag: #todo. "access the repository in another way"

	^ ((((FLFameObjectClusterSerializer instance repository metamodel descriptionOf: aClass) allComplexAttributes)
		reject: [ :anAttribute | anAttribute isDerived ])
		collect: [ :anAttribute | anAttribute type implementingClass ]) asSet! !

!FLFameAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 11/23/2010 11:46'!
clusteredFameObjects
	
	^ clusteredFameObjects! !

!FLFameAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 11/23/2010 18:10'!
clusteredObjects
	"Returns a new collection with all the clustered objects, in Association instances."

	^ super clusteredObjects
		addAll: self sortedClusteredFameObjects;
		yourself.! !

!FLFameAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 11/24/2010 11:39'!
initialize

	super initialize.
	clusteredFameObjects := Dictionary new.! !

!FLFameAnalyzer methodsFor: 'initialize-release' stamp: 'MartinDias 11/24/2010 11:34'!
initializeDefaultAnalyzers

	super initializeDefaultAnalyzers.
	analyzers addFirst: (FLFameObjectAnalyzer newWith: self)! !

!FLFameAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 11/24/2010 01:58'!
lessDependedOf: associations

	^ associations detect: [ :anAssoc | 
		associations allSatisfy: [ :anotherAssoc | 
			anAssoc == anotherAssoc or: [ 
				((self classDependenciesOf: anAssoc key clusteredClass) 
					anySatisfy: [ :aClass | anotherAssoc key clusteredClass isKindOf: aClass ]) not ] ] ]! !

!FLFameAnalyzer methodsFor: 'accessing' stamp: 'MartinDias 11/24/2010 11:31'!
sortedClusteredFameObjects

	| sortedAssociations yetNotSortedAssociations |
	sortedAssociations := OrderedCollection new.
	yetNotSortedAssociations := clusteredFameObjects associations asOrderedCollection.

	[ yetNotSortedAssociations isEmpty ] whileFalse: [ 
		| next |
		next := self lessDependedOf: yetNotSortedAssociations..
		sortedAssociations add: next.
		yetNotSortedAssociations remove: next ].

	^ sortedAssociations! !

TestCase subclass: #FLFameAnalyzerTest
	instanceVariableNames: 'analyzer repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Tests'!

!FLFameAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 11/24/2010 01:53'!
assert: aClass notDependsOn: yetNotVisitedClasses

	((repository metamodel descriptionOf: aClass) allComplexAttributes
		reject: [ :anAttribute | anAttribute isDerived ])
			do: [ :anAttribute | 
				self deny: (yetNotVisitedClasses anySatisfy: [ :aYetNotVisitedClass | 
					aYetNotVisitedClass isKindOf: anAttribute type implementingClass ]) ]! !

!FLFameAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 11/22/2010 17:50'!
repositoryWithSample
	
	^ (FMRepository with: MooseModel meta) 
			addAll: self sampleMooseModel entities;
			yourself.! !

!FLFameAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 2/23/2011 00:52'!
sampleMooseModel

	^ MooseSqueakClassPackageImporterTask new
		model: MooseModel new;
		addFromPackageNamed: 'Merlin';
		run;
		model.! !

!FLFameAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 11/24/2010 11:36'!
setUp

	super setUp.
	repository := self repositoryWithSample.
	FLFameObjectClusterSerializer instance repository: repository.
	analyzer := FLFameAnalyzer newDefault! !

!FLFameAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 2/23/2011 03:41'!
testClusteredObjectsOrder
	"The associations in clusteredObjects should be ordered ... (complete)"
	
	| yetNotVisitedClasses describedClasses |
	analyzer analyze: self sampleMooseModel entities.

	describedClasses := ((analyzer clusteredObjects 
		select: [ :anAssociation | 
			anAssociation key isKindOf: FLFameObjectClusterSerializer ]) 
		collect: [ :anAssociation | 
			anAssociation key clusteredClass ]).
	
	self assert: describedClasses size >= 2 description: 'Case has not sense if classes are less than 2'.
	
	yetNotVisitedClasses := describedClasses copy.
	
	describedClasses
		do: [ :aClass | 
			yetNotVisitedClasses removeFirst.
			self assert: aClass notDependsOn: yetNotVisitedClasses]
 ! !

TestCase subclass: #FLFameObjectAnalyzerTest
	instanceVariableNames: 'mooseModel sampleDescription repository'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Tests'!

!FLFameObjectAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 11/25/2010 18:42'!
descriptionOfSampleIn: aRepository
		
	^ aRepository elements detect: [ :x | x class = FAMIXClass and: [ x name = #FLPair] ]! !

!FLFameObjectAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 11/16/2010 23:53'!
repositoryWithSample
	
	^ (FMRepository with: MooseModel meta) 
			addAll: self sampleMooseModel entities;
			yourself.! !

!FLFameObjectAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 11/22/2010 14:54'!
sampleAnalyzer
	^ FLFameObjectAnalyzer newWith: nil! !

!FLFameObjectAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 11/25/2010 18:41'!
sampleMooseModel

	^ MooseSqueakClassPackageImporterTask new
		model: MooseModel new;
		addClass: FLPair;
		run;
		model.
! !

!FLFameObjectAnalyzerTest methodsFor: 'running' stamp: 'MartinDias 11/17/2010 01:37'!
setUp

	super setUp.
	repository := self repositoryWithSample.
	FLFameObjectClusterSerializer instance repository: repository.
	sampleDescription := self descriptionOfSampleIn: repository.
! !

!FLFameObjectAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 11/17/2010 00:41'!
testAnalyzes

	self
		assert: (self sampleAnalyzer analyzes: sampleDescription)
		description: 'It should accept analyzing an object whose class is described in the repository'! !

!FLFameObjectAnalyzerTest methodsFor: 'tests' stamp: 'MartinDias 11/17/2010 00:42'!
testDoesNotAnalyze

	self 
		deny: (self sampleAnalyzer analyzes: 42) 
		description: 'It should not accept analyzing an object whose class is not described in the repository'! !

!FLMaterializer methodsFor: '*FuelFameExtension' stamp: 'MartinDias 2/26/2011 05:18'!
materializeEntitiesDescribedWith: aMetamodel
	"Materializes Fame entities from the stream, and returns that result"

	| entities tower model |
	self flag: #todo. "Some of these steps are here just to imitate the MSE behaviour but could be not needed"
	tower := FMCompleteTower new.
	tower metamodel addAll: aMetamodel elements.

	FLFameObjectClusterSerializer instance repository: tower model.
	entities := self materialize; root.

	tower model addAll: entities.
	tower model updateCache.
	model := MooseModel new.
	tower model elements do: [ :e | model add: e ].
	model entityStorage forRuntime.
	^ model
! !

FLClusterSerializer subclass: #FLFameObjectClusterSerializer
	instanceVariableNames: 'repository clusteredClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelFameExtension-Core'!

!FLFameObjectClusterSerializer commentStamp: '<historical>' prior: 0!
I serialize and materialize objects using its Famix meta description. 

At the moment, the metamodel should be the same when serializing and materializing, or incongruences could appear.!

!FLFameObjectClusterSerializer classSide methodsFor: 'instance creation' stamp: 'MartinDias 11/17/2010 01:01'!
for: aClass

	^ self basicNew
		clusteredClass: aClass;
		yourself.! !

!FLFameObjectClusterSerializer classSide methodsFor: 'accessing' stamp: 'MartinDias 11/16/2010 16:12'!
id

	^ 82! !

!FLFameObjectClusterSerializer methodsFor: 'comparing' stamp: 'MartinDias 11/17/2010 01:02'!
= anObject
	"Answer whether the receiver and anObject represent the same object."

	self == anObject
		ifTrue: [ ^ true ].
	self class = anObject class
		ifFalse: [ ^ false ].
	^ self clusteredClass = anObject clusteredClass! !

!FLFameObjectClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 11/17/2010 01:03'!
clusteredClass
	^ clusteredClass! !

!FLFameObjectClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 11/17/2010 01:03'!
clusteredClass: anObject
	clusteredClass := anObject! !

!FLFameObjectClusterSerializer methodsFor: 'comparing' stamp: 'MartinDias 11/17/2010 01:02'!
hash
	"Answer an integer value that is related to the identity of the receiver."

	^ self clusteredClass hash! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 2/19/2011 02:28'!
materializeAllWith: aMaterializer

	| class instanceCount clusterInstances classDescription attributes |
	class := aMaterializer nextReferencedObject.
	instanceCount := aMaterializer nextInteger.

	classDescription := repository metamodel descriptionOf: class.
	clusterInstances := OrderedCollection new: instanceCount.
	instanceCount timesRepeat: [ clusterInstances add: classDescription createInstance ].

	attributes := classDescription allAttributes reject: [ :anAttribute | anAttribute isDerived ].

 	aMaterializer 
		addAllMaterialized: clusterInstances;
		referenceMaterializingActionsAdd:
			[ self 
				materializeAttributes: attributes
				ofAll: clusterInstances 
				with: aMaterializer 
				classDescription: classDescription ]! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/27/2010 15:23'!
materializeAttribute: anInstance of: anAttribute with: aMaterializer

	| valuesSize values |
	valuesSize := aMaterializer nextInteger.

	self flag: #todo. "Encoding for skip ignored values (see #serializeIgnoredValueWith:)"
	(valuesSize = 0) 
		ifTrue: [ ^ self ].

	values := Array new: valuesSize.
	1 to: valuesSize do: [ :index | 
		values at: index put: aMaterializer nextReferencedObject ].	
		
	anAttribute 
		setOn: anInstance 
		values: values! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/22/2010 11:08'!
materializeAttributes: attributes ofAll: instances with: aMaterializer classDescription: aClassDescription

	instances do: [ :anInstance |
		attributes do: [ :anAttribute |
			self 
				materializeAttribute: anInstance 
				of: anAttribute 
				with: aMaterializer ] ]! !

!FLFameObjectClusterSerializer methodsFor: 'printing' stamp: 'MartinDias 11/17/2010 15:30'!
printOn: aStream

	super printOn: aStream.
	aStream 
		nextPutAll: ' (';
		nextPutAll: clusteredClass fullName;
		nextPutAll: ')'.! !

!FLFameObjectClusterSerializer methodsFor: 'analyzing' stamp: 'MartinDias 11/24/2010 14:37'!
referencesOf: anObject do: aBlock

	| description |
	aBlock value: anObject class.

	((repository metaDescriptionOf: anObject) 
		allAttributes reject: [ :anAttribute | anAttribute isDerived ]) 
			do: [ :anAttribute |
				(repository 
					get: anAttribute
					element: anObject) 
						do: [ :aValue |
							aBlock value: aValue ] ]! !

!FLFameObjectClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 11/16/2010 23:43'!
repository
	^ repository! !

!FLFameObjectClusterSerializer methodsFor: 'accessing' stamp: 'MartinDias 11/16/2010 23:43'!
repository: anObject
	repository := anObject! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 2/19/2011 02:29'!
serializeAll: clusterInstances with: aSerializer

	| attributes |
	attributes := (repository metamodel descriptionOf: clusteredClass) allAttributes reject: [ :anAttribute | anAttribute isDerived ].
	
	aSerializer 
		nextPutReferenceTo: clusteredClass;
		nextPutInteger: clusterInstances size;
		referenceSerializingActionsAdd:
			[ self serializeAttributes: attributes ofAll: clusterInstances with: aSerializer ]! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/24/2010 10:24'!
serializeAttribute: anAttribute values: values with: aSerializer

	(self shouldIgnore: anAttribute withAll: values)
		ifTrue: [ self serializeIgnoredValueWith: aSerializer ]
		ifFalse: [ self serializeValues: values with: aSerializer ]
! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/22/2010 10:59'!
serializeAttributes: attributes ofAll: instances with: aSerializer

	instances do: [ :anInstance |
		attributes do: [ :anAttribute |
			self 
				serializeAttribute: anAttribute 
				values: (repository get: anAttribute element: anInstance) 
				with: aSerializer ] ].! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 12/27/2010 15:22'!
serializeIgnoredValueWith: aSerializer

	self flag: #todo. "Encode in another way that its an ignored value."
	aSerializer nextPutInteger: 0! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/17/2010 11:38'!
serializeValues: values with: aSerializer

	aSerializer nextPutInteger: values size.
	values do: [ :aValue | aSerializer nextPutReferenceTo: aValue ]! !

!FLFameObjectClusterSerializer methodsFor: 'serialize/materialize' stamp: 'MartinDias 11/24/2010 10:22'!
shouldIgnore: attribute withAll: values
	"Copied from FMRepositoryVisitor>>ignoreProperty:withAll:"

	^ values isEmpty or: [
		attribute isDerived or: [
			attribute type == FM3MetaDescription boolean and: [
				values size == 1 and: [ 
					values first == false ]]]]! !
